#
# pymldb
# Copyright (c) 2013 Datacratic. All rights reserved.
#

from pymldb import resource
import pandas as pd


class Connection(object):
    
    def __init__(self, host="http://localhost"):
        if not host.startswith("http"): 
            raise Exception("URIs must start with 'http'")
        self.host = host.strip("/")
        self.v1 = resource.Resource(self.host).v1
        
    def query(self, sql):
        resp = self.v1.query.get(params=dict(q=sql, format="aos"), raise_on_error=False)
        if resp.status_code != 200: return resp
        resp_json = resp.json()
        if len(resp_json) == 0: 
            return pd.DataFrame()
        else:
            return pd.DataFrame.from_records(resp_json, index="_rowName")
    
    #def batframe(self, dataset_id):
    #    return data.BatFrame(self.v1.datasets(dataset_id).uri)


# IPython Magic system

def load_ipython_extension(ipython, *args):
    from pymldb.magic import dispatcher
    dispatcher("init http://localhost")
    ipython.register_magic_function(dispatcher, 'line_cell', magic_name="mldb")

def unload_ipython_extension(ipython):
    pass



import requests, json
from pymldb.util import add_repr_html_to_response

class ResourceError(Exception):
    def __init__(self, r):
        try:
            message = json.dumps(r.json(), indent=2)
        except:
            message = r.content
        super(ResourceError, self).__init__(
            "'%d %s' response to '%s %s'\n\n%s" %
            (r.status_code, r.reason, r.request.method, r.request.url,message)
        )
        self.result = r

def decorate_response(fn):
    def inner(*args, **kwargs):
        raise_on_error = True
        if "raise_on_error" in kwargs:
            raise_on_error = kwargs["raise_on_error"]
            del kwargs["raise_on_error"]
        result = add_repr_html_to_response(fn(*args, **kwargs))
        if raise_on_error and result.status_code >= 400:
            raise ResourceError(result)
        return result
    return inner

class Resource(object):
    def __init__(self, uri="http://localhost"):
        if not uri.startswith("http"):
            raise Exception("URIs must start with 'http'")
        self.uri = uri.strip("/")

    def __repr__(self):
        return self.uri

    def __str__(self):
        return self.uri

    def __call__(self, frag):
        return Resource(self.uri+"/"+str(frag).strip("/"))

    def __getattr__(self, frag):
        return Resource(self.uri+"/"+str(frag).strip("/"))

    @decorate_response
    def get(self, **kwargs):
        return requests.get(self.uri,
            params= {k: (json.dumps(v) if isinstance(v, dict) else v)
                 for k,v in kwargs.items() })

    @decorate_response
    def put(self, payload):
        return requests.put(self.uri, json=payload)

    @decorate_response
    def post(self, payload):
        return requests.post(self.uri, json=payload)

    @decorate_response
    def delete(self):
        return requests.delete(self.uri)
